import discord
import discord.ext.commands as commands


class Roles(commands.Cog):
    def __init__(self, bot: discord.ext.commands.Bot, role_id, message_id, channel_id, emoji_id):
        self.bot = bot
        self.role_id = role_id
        self.message_id = message_id

        emoji = self.bot.get_emoji(emoji_id)
        self.role_emojis = {
            emoji: role_id
        }

        channel = self.bot.get_channel(channel_id)
        message = channel.fetch_message(message_id)
        message.add_reaction(emoji)

    @commands.Cog.listener()
    async def on_raw_reaction_add(self, payload: discord.RawReactionActionEvent):
        role = self.get_role_from_emoji(payload)

        try:
            # Finally, add the role.
            await payload.member.add_roles(role)
        except discord.HTTPException:
            # If we want to do something in case of errors we'd do it here.
            pass

    @commands.Cog.listener()
    async def on_raw_reaction_remove(self, payload: discord.RawReactionActionEvent):
        """Removes a role based on a reaction emoji."""

        role = self.get_role_from_emoji(payload)

        # The payload for `on_raw_reaction_remove` does not provide `.member`
        # so we must get the member ourselves from the payload's `.user_id`.
        guild = self.bot.get_guild(payload.guild_id)
        if guild is None:
            # Check if we're still in the guild and it's cached.
            return

        member = guild.get_member(payload.user_id)
        if member is None:
            # Make sure the member still exists and is valid.
            return

        try:
            # Finally, remove the role.
            await member.remove_roles(role)
        except discord.HTTPException:
            # If we want to do something in case of errors we'd do it here.
            pass

    def get_role_from_emoji(self, payload: discord.RawReactionActionEvent):
        # Ignore reactions given by bot
        if payload.user_id == self.bot.user.id:
            return

        # Make sure that the message the user is reacting to is the one we care about.
        if payload.message_id != self.message_id:
            return

        # The payload for `on_raw_reaction_remove` does not provide `.member`
        # so we must get the member ourselves from the payload's `.user_id`.
        guild = self.bot.get_guild(payload.guild_id)
        if guild is None:
            # Check if we're still in the guild and it's cached.
            return

        try:
            role_id = self.role_emojis[payload.emoji]
        except KeyError:
            # If the emoji isn't the one we care about then exit as well.
            return

        role = guild.get_role(role_id)
        if role is None:
            # Make sure the role still exists and is valid.
            return

        return role
