import json
import os


# TODO Check for malformed or missing entries
class ConfigDict(dict):
    """Provides config persistence by writing to config.json"""

    def __init__(self):
        super().__init__()
        self.config_dir = os.path.join(os.path.expanduser("~"), ".config/ntpubbot")
        self.config_file = os.path.join(self.config_dir, "config.json")

        if os.path.exists(self.config_dir) is False:
            os.mkdir(self.config_dir)

        if os.path.isfile(self.config_file):
            self._read_from_file()
        else:
            # Set initial key-values to provide example config.json after write
            self.update({
                "min_pop": 1,
                "timeout": 3600,
                "role_id": 838000652709396491,
                "channel_id": 765851775122931742,
                "message_id": 0,
                "emoji_id": 838210540185845800,
                "discord_token": ""
            })
            self._write_to_file()

    def __setitem__(self, key, value):
        super().__setitem__(key, value)
        self._write_to_file()

    def _write_to_file(self):
        # TODO Write only modified entries instead of whole config
        try:
            with open(self.config_file, "w") as file:
                json.dump(self, file)
        except IOError:
            print("Couldn't write config. Config file doesn't exist")

    def _read_from_file(self):
        try:
            with open(self.config_file, "r") as file:
                self.update(json.load(file))
        except IOError:
            print("Couldn't read config. Config file doesn't exist")


def main():
    # Call constructor to generate template config.json
    ConfigDict()
